#ifndef	DVTICKET_CGI_H
#define	DVTICKET_CGI_H
// $Id: cgi.h,v 1.7 2003/08/16 10:10:23 dvermeir Exp $

#include <stdexcept>
#include <dvcgi/cgi.h>
#include <dvticket/ticket.h>

namespace Dv {
namespace Ticket {

/** A specialization of Dv::Cgi::Cgi, which transparently verifies
 * the user identity, using a login.cgi server. One of the
 * advantages of this class is that the cgi program never needs
 * to handle or know passwords.
 */
class Cgi: public Dv::Cgi::Cgi {
public:
   /** Constructor. Upon success, the identity of the use is known.
    * @param name of application.
    * @param login_url (suffix) where login server runs. Should be such
    *  that https://login_url is a well-formed (and valid!) url.
    * @param default_language 2-letter ISO code of default language to
    *  be used (may be switched by user when logging in via login.cgi)
    * @param ssl_only if true, non-SSL connections will be redirected
    *  to the https-version of the same url.
    * @param type mime type to be returned by this cgi program.
    * @param debug if true, header() will have debug flag set.
    */
  Cgi(const std::string& name, const std::string& login_url, 
    const std::string& default_language, bool ssl_only=false,
    const std::string& type="text/html", bool debug=false) throw ();
  /** Destructor. */
  ~Cgi();

  /** @return Ticket or 0 if none. */
  Dv::Ticket::Ticket::Ref  ticket() const { return ticket_; }
  /** @return user owning ticket corresponding to cookie. */
  const Dv::Ticket::User&  user() const { return ticket_->user(); }
  /** @return true iff we were called using https. */
  bool  is_ssl() const { return is_ssl_; }
  /** @return value of language cookie, default is "nl". */
  const std::string&  language() const { return language_; }

  
private:
  /** the ticket, identifying an authenticated user (or, possibly,
   * an anonymous user), associated with this CGI object */
  Dv::Ticket::Ticket::Ref  ticket_;
  /** true iff this cgi program must be accessed only using SSL connections */
  bool  is_ssl_;
  /** language to communicate with the user */
  std::string  language_;
}; 

}}
#endif
