#ifndef DV_TICKET_TEST_AUTHENTICATOR
#define DV_TICKET_TEST_AUTHENTICATOR
// $Id: test-authenticator.h,v 1.3 2003/07/23 08:38:13 dvermeir Exp $
#include <stdexcept>
#include <dvticket/user.h>
#include <dvticket/authenticator.h>
#include "test-trusted-ip-collection.h"

namespace Dv {
namespace Ticket {
/**
 * A function object class that can check a user's identity,
 * based on a login name and password.
 * This is an abstract class. An application should implement
 * a derived class. The default implementation always succeeds.
 *
 * @see Dv::Ticket::Request
 */
class TestAuthenticator: public Authenticator {
public:
  /** Constructor.
   * @param name of file containing trusted IPs.
   */
  TestAuthenticator(const std::string& fn) throw (std::runtime_error):
    ip_collection(fn) {}
  /** Destructor. */
  virtual ~TestAuthenticator() {}
 
  /** @return name of this Authenticator.
   * Derived classes should have different names, which will
   * show up in the server logs.
   * @sa Dv::Ticket::Server
   */
  virtual std::string const name() { return "TestAuthenticator"; }

  /** Actual authentication of a user.
   * @param host from which user requests authentication
   * @param name login name of the user.
   * @param passwd password of the user.
   * @param info extra info on user.
   * @exception runtime_error if there is no user
   *   corresponding to the @a name and @a passwd.
   */
  virtual Dv::Ticket::User validate(const std::string& host, const std::string& name,
    const std::string& passwd, const std::string& info="") 
      throw (std::runtime_error) { return Dv::Ticket::User("student", 10, name, info); }

  /** Return an anonymous user.
   * @param host from which user requests authentication
   * @param name login name of the user.
   * @param info extra info on user.
   * @exception runtime_error if there is no user
   *   corresponding to the @a name and @a passwd.
   */
  virtual Dv::Ticket::User anon_validate(const std::string& host, const std::string& name,
    const std::string& info="") throw (std::runtime_error) {
      if ( ip_collection.trusted(host) )
        return Dv::Ticket::User("anonymous", 0, name, info); 
      throw std::runtime_error("FAIL");
      }
private:
  TrustedIpCollection ip_collection;
};

}}
#endif

