// $Id: test-xml-check-server.C,v 1.3 2003/08/08 09:51:00 dvermeir Exp $
#include <unistd.h> // for fork(), sleep()
#include <sys/types.h> // wait()
#include <sys/wait.h> // wait()
#include <dvutil/file.h>
#include <dvutil/systemerror.h>
#include <dvnet/socket.h>
#include <dvnet/serversocket.h>
#include <dvxml/xml.h>

#include "xml-sndrcv.h"

static const std::string data_dir(DATADIR);
static const std::string request_fn(data_dir + "/test-request.xml");
static const std::string dtd_fn(data_dir + "/request.dtd");

///////////////////////////////////////////////////////////////////////////////////////////////
// Send an XML document and receive it back from the server, return true
// if all went well.
///////////////////////////////////////////////////////////////////////////////////////////////
bool
echo_xml(const std::string& host, int port) {

std::cerr << "client: connecting to " << host << ":" << port << std::endl;
try {
  Dv::Net::Socket socket(host, port);

  if (!socket) 
    throw std::runtime_error(socket.strerror() + ": connection failed");

  Dv::Util::File request_f(request_fn);
  Dv::Xml::Document sent_doc(request_f);
  if (! sent_doc.valid(dtd_fn) )
    throw std::runtime_error("sent_doc not valid");

  Dv::Xml::send(sent_doc, socket);

  // receive a document
  Dv::Xml::Document received_doc;

  Dv::Xml::receive(received_doc, socket);
  if (! received_doc.valid(dtd_fn) )
    throw std::runtime_error("received_doc not valid");
  std::cerr << "client: done" << std::endl;
  return true;
  }
catch (std::exception& e) {
  std::cerr << "client: " << e.what() << std::endl;
  return false;
  }
}

///////////////////////////////////////////////////////////////////////////////////////////////
int
main(int argc,char* argv[]) {

try {
  std::cout << "request_fn = " << request_fn
  	    << ", dtd_fn = " << dtd_fn 
	    << std::endl;

  Dv::Util::File request_f(request_fn);
  if (! request_f.exists())
    throw std::runtime_error(request_fn + ": cannot open");
  Dv::Util::File dtd_f(dtd_fn);
  if (! dtd_f.exists())
    throw std::runtime_error(dtd_fn + ": cannot open");

  Dv::Util::ref<Dv::Net::InetAddress> ia(Dv::Net::InetAddress::by_name("localhost"));

  if (!ia) 
    throw std::runtime_error("Cannot determine InetAddress of local host");

  int port(9999);

  std::cerr << "main: server on " << ia->host() << ":" << port << std::endl;


  // The parent will be the server, the child the client.
  int child = fork();

  if (child<0)
    throw Dv::Util::SystemError();
  
  ////////////////////////////////////////////////////////////////////////////////////////////////
  // Client fork
  if (child==0) {
    std::cerr << "client: sleeping .." << std::endl;
    sleep(2);
    std::cerr << "client: starting .." << std::endl;
    return ( echo_xml(ia->host(), port) ? 0 : 1 );
    }
  //
  ////////////////////////////////////////////////////////////////////////////////////////////////

  Dv::Net::ServerSocket server(port);
  std::cerr << "server: start.." << std::endl;
  
  Dv::Util::ref<Dv::Net::Socket> client( server.accept() );
  
  std::cerr << "server: accepted connection from "
            << client->host() << ":" << client->port() << std::endl;

  Dv::Xml::Document document;
  Dv::Xml::receive(document, *client);
  std::cerr << "server: received document: " << std::endl << document << std::endl;
  if (! document.valid(dtd_fn) )
    throw std::runtime_error("received document not valid");
  Dv::Xml::send(document, *client);
  std::cerr << "server: sent document: " << std::endl;

  int client_status;
  wait(&client_status);

  if (WIFEXITED(client_status)) 
     if (WEXITSTATUS(client_status)==0)
       return 0;
     else
       return 1;
  else
    return 1;
  }
catch (std::exception& e) {
  std::cerr << "server: " << e.what() << std::endl;
  return 2;
  }
}
