// $Id: test-xml-server.C,v 1.1 2003/07/09 12:46:14 dvermeir Exp $
#include <unistd.h> // for fork(), sleep()
#include <sys/types.h> // wait()
#include <sys/wait.h> // wait()
#include <dvutil/systemerror.h>
#include <dvnet/socket.h>
#include <dvnet/serversocket.h>
#include <dvxml/xml.h>

#include "xml-sndrcv.h"

///////////////////////////////////////////////////////////////////////////////////////////////
// Send an XML document and receive it back from the server, return true
// if all went well.
///////////////////////////////////////////////////////////////////////////////////////////////
bool
echo_xml(const std::string& host, int port) {

static const std::string xml_text("<?xml version=\"1.0\"?>"
  "<bla> <ohlala>ohlala</ohlala> </bla>" );

std::cerr << "client: connecting to " << host << ":" << port << std::endl;
try {
  Dv::Net::Socket socket(host, port);

  if (!socket) 
    throw std::runtime_error(socket.strerror() + ": connection failed");

  Dv::Xml::Document sent_doc( xml_text.data(), xml_text.size() );

  std::string sent_string;
  sent_doc.save_to_string(sent_string);
  std::cerr << "client: document sent: " << sent_string << std::endl;

  Dv::Xml::send(sent_doc, socket);

  // receive a document
  Dv::Xml::Document received_document;
  std::string received_string;

  Dv::Xml::receive(received_document, socket);
  received_document.save_to_string(received_string);
  std::cerr << "client: document received: " << received_string << std::endl;

  // compare both documents
  return sent_string == received_string;
  }
catch (std::exception& e) {
  std::cerr << "client: " << e.what() << std::endl;
  return false;
  }
}

///////////////////////////////////////////////////////////////////////////////////////////////
int
main(int argc,char* argv[]) {

try {
  Dv::Util::ref<Dv::Net::InetAddress> ia(Dv::Net::InetAddress::by_name("localhost"));

  if (!ia) 
    throw std::runtime_error("Cannot determine InetAddress of local host");

  int port(9999);

  std::cerr << "main: server on " << ia->host() << ":" << port << std::endl;


  // The parent will be the server, the child the client.
  int child = fork();

  if (child<0)
    throw Dv::Util::SystemError();
  
  ////////////////////////////////////////////////////////////////////////////////////////////////
  // Client fork
  if (child==0) {
    std::cerr << "client: sleeping .." << std::endl;
    sleep(2);
    std::cerr << "client: starting .." << std::endl;
    return ( echo_xml(ia->host(), port) ? 0 : 1 );
    }
  //
  ////////////////////////////////////////////////////////////////////////////////////////////////

  Dv::Net::ServerSocket server(port);
  std::cerr << "server: start.." << std::endl;
  
  Dv::Util::ref<Dv::Net::Socket> client( server.accept() );
  
  std::cerr << "server: accepted connection from "
            << client->host() << ":" << client->port() << std::endl;

  Dv::Xml::Document document;
  Dv::Xml::receive(document, *client);
  std::cerr << "server: received document: " << document << std::endl;
  Dv::Xml::send(document, *client);

  int client_status;
  wait(&client_status);

  if (WIFEXITED(client_status)) 
     if (WEXITSTATUS(client_status)==0)
       return 0;
     else
       return 1;
  else
    return 1;
  }
catch (std::exception& e) {
  std::cerr << "server: " << e.what() << std::endl;
  return 2;
  }
}
